/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file dn_mapper.h
 * @brief Mapping from DNs to usernames.
 *
 * This file implements mapping from distinguished names to a string that can
 * be used as a local UNIX username without conflicts. Different mapping
 * strategies can be implemented as modules. */
#ifndef __DN_MAPPER_H__
#define __DN_MAPPER_H__

#include <stddef.h>		/* size_t */

/** @brief Module description structure.
 *
 * This structure holds pointers to the different functions of a mapper
 * module */
struct _nss_xos_dn_mapper_module {
	/** Check whether name corresponds to a distinguished name */
	int (*is_map) (const char *name);
	/** Translate a DN into a UNIX username and place the resulting string
	 * in @a buf */
	 size_t(*dn2name) (const char *dn, char *buf, size_t bufsiz);
	/** Translate a UNIX username into a DN and place the resulting string
	 * in @a buf */
	 size_t(*name2dn) (const char *name, char *buf, size_t bufsiz);
};

/** Check whether the mapped DN stored in buf actually corresponds to a DN.
 * @param buf Buffer storing the mapped dn
 * @retval 0 @a buf is not a DN
 * @retval !0 @a buf is a DN
 */
int _nss_xos_dn_mapper_is_map(const char *name);

/** Map a distinguished name into a UNIX username according to the current
 * mapper module.
 * @param dn Distinguished name
 * @param buf Buffer to store the mapped dn
 * @param bufsiz Size of @a buf
 * @return Used buffer, or 0 if error
 */
size_t _nss_xos_dn_mapper_dn2name(const char *dn, char *buf,
				  size_t bufsiz);

/** Map a UNIX username into a distinguished name according to the current
 * mapper module.
 * @param name UNIX username
 * @param buf Buffer to store the mapped name
 * @param bufsiz Size of @a buf
 * @return Used buffer, or 0 if error
 */
size_t _nss_xos_dn_mapper_name2dn(const char *name, char *buf,
				  size_t bufsiz);

#endif				/* __DN_MAPPER_H__ */
