/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/**
 * @file libnss_xos.h
 * @brief NSS-related definitions.
 *
 * This file should only be included by the NSS module.
 */
#ifndef __LIBNSS_XOS_H__
#define __LIBNSS_XOS_H__

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif				/* HAVE_CONFIG_H */

#include <errno.h>		/* ERANGE */
#include <grp.h>		/* struct group */
#include <nss.h>		/* enum nss_status */
#include <pwd.h>		/* struct passwd */
#include <string.h>		/* strcpy */
#include <sys/types.h>		/* size_t */

#include "xos_utils.h"		/* DEBUG */

/**
 * @def NSS_DEBUG(args...)
 * This macro outputs a debug message if debug is set to true.
 */
#if ENABLE_DEBUG_NSS
# define NSS_DEBUG(args...) do { \
	if (_nss_xos_opts.debug) \
		DEBUG(args); \
} while (0)
#else				/* ENABLE_DEBUG_NSS */
# define NSS_DEBUG(args...) ((void)(0))
#endif				/* ENABLE_DEBUG_NSS */

#define NSS_LOG(level, args...) \
	xos_log(LOG_PID | LOG_CONS, LOG_AUTHPRIV, level, args)

/** NSS return status */
#define NSS_STATUS enum nss_status
/** Parameters received by passwd fuctions */
#define PW_PARAMS struct passwd *pw
/** Parameters received by group fuctions */
#define GR_PARAMS struct group *gr
/** Common parameters */
#define DEF_PARAMS char *buffer, size_t buflen, int *errnop

/** Move data into @a buffer checking for overflow.
 * @param[in,out] buffer Buffer where \a val will be copied
 * @param[in,out] buflen Size of \a buffer
 * @param[out]    dest A <tt>char *</tt> that will be set to the place in @a
 *                     buffer where @a val has been copied (usually the
 *                     corresponding <tt>struct passwd</tt> field
 * @param         val Value (string) to copy into the buffer
 * @param[out]    errnop Pointer to the error that will be returned in case of
 *                       error
 */
#define BUF_CHR(buffer, buflen, dest, val, errnop) do { \
	size_t len = strlen(val) + 1; \
	if (len <= buflen) { \
		strcpy(buffer, val); \
		dest = buffer; \
		buffer += len; \
		buflen -= len; \
	} else { \
		if (_nss_xos_opts.debug) DEBUG("BUF_CHR: Buffer overflow!"); \
		*errnop = ERANGE; \
		/* libc will call again with a bigger buffer size */ \
		nss_ret = NSS_STATUS_TRYAGAIN; \
		goto buff_overflow; \
	} \
} while(0)

/** Move @b mapped data into @a buffer checking for overflow.
 * @param         func The function _nss_xos_dn_mapper_@a func will be used to
 *                     map @a val before copying it into the buffer
 * @param[in,out] buffer Buffer where the mapped \a val will be copied
 * @param[in,out] buflen Size of \a buffer
 * @param[out]    dest A <tt>char *</tt> that will be set to the place in @a
 *                     buffer where the mapped @a val has been copied (usually
 *                     the corresponding <tt>struct passwd</tt> field
 * @param         val Value (string) to copy into the buffer
 * @param[out]    errnop Pointer to the error that will be returned in case of
 *                       error
 */
#define BUF_MAP(func, buffer, buflen, dest, val, errnop) do { \
	size_t len = _nss_xos_dn_mapper_ ## func(val, buffer, buflen); \
	if (len != 0) { \
		dest = buffer; \
		buffer += len; \
		buflen -= len; \
	} else { /* error */ \
		if (_nss_xos_opts.debug) DEBUG("BUF_MAP: Buffer overflow!"); \
		*errnop = ERANGE; \
		/* libc will call again with a bigger buffer size */ \
		nss_ret = NSS_STATUS_TRYAGAIN; \
		goto buff_overflow; \
	} \
} while(0)

/** Skip @a len bytes in @a buffer checking for overflow (this macro is needed
 * for group mappings, to leave room for members' UIDs).
 * @param[in,out] buffer Buffer where @a len bytes of space will be skipped
 * @param[in,out] buflen Size of @a buffer
 * @param         len Number of bytes to skip
 * @param[out]    errnop Pointer to the error that will be returned in case of
 *                       error
 */
#define BUF_SKIP(buffer, buflen, len, errnop) do { \
	if (len <= buflen) { \
		buffer += len; \
		buflen -= len; \
	} else { \
		if (_nss_xos_opts.debug) DEBUG("BUF_SKIP: Buffer overflow!"); \
		*errnop = ERANGE; \
		/* libc will call again with a bigger buffer size */ \
		nss_ret = NSS_STATUS_TRYAGAIN; \
		goto buff_overflow; \
	} \
} while(0)

/** Mapping scheme to translate DNs. */
enum mapping_scheme {
	MAP_SCH_RAWDN = 0,	/**< No mapping (use raw DN) */
	MAP_SCH_SCAPECHAR	/**< Map special characters to "x<HEX CODE>" */
};

/** Access to database modes. */
enum db_access_mode {
	DB_ACCESS_ALL = 0,	/**< All users (local and global) have access to
				     the information stored in the database */
	DB_ACCESS_ROOT,		/**< Only global users and the local root have
				     access to the information stored in the
				     database. Local unprivileged users are
				     denied access to the database. */
	DB_ACCESS_RESTRICTED	/**< Only global users have access to the
				     database. Local users are denied access to
				     the database. */
};

/** NSSwitch module configuration options. */
struct _nss_xos_config_options {
	int debug;		/**< Debug mode (0: disabled, 1: enabled) */
	enum mapping_scheme m_scheme;	/**< Mapping scheme for DNs */
	enum db_access_mode db_access;	/**< Database access mode */
};

/**
 * @brief Variable to cache configuration options (protected by a mutex).
 *
 * This variable can be directly accessed by the NSSwitch module provided that
 * _nss_xos_loadconf has previously been called.
 */
extern struct _nss_xos_config_options _nss_xos_opts;

/** @brief Load configuration options.
 * @param[out] opts configuration options variable (must be _nss_xos_opts)
 * @retval 0 if success
 * @retval -1 if failure
 */
int _nss_xos_loadconf(struct _nss_xos_config_options *opts);

/** @brief Check if user has access to the database.
 * @param opts configuration options (must have been previously initialized by
 *        calling @a _nss_xos_loadconf())
 * @retval 0 if access to the database is not allowed
 * @retval 1 if access to the database is allowed
 */
int _nss_xos_db_access_allowed(const struct _nss_xos_config_options *opts);

#endif				/* __LIBNSS_XOS_H__ */
