/** @file xos_dbaux.h
 * @brief Data structure of user's information, Interfaces of database
 *
 * This file presents definition of data structures about Grid user informaton,
 * mapping information. All the information will be devided into two categories:
 * a) user's account information defined by struct \a GPASSWD, in which 
 * the information includes user's global identity token (DN,VO,ROLE), local mapped
 * identity token (name,uid,primary-gid,home-dir,shell).
 * b) user's group information defined by struct \a GGROUPS, in which the information
 * includes user's VO identity and VO subgroups, and also local mapped group 
 * information (group name, gid, members).
 * Each structure (\a GPASSWD and \a GGROUPS) is representative of one mapping 
 * relationship. A time stamp is attached to each structure for logging and auditting.
 *
 * The databases are also divied into two categories: one for storing GPASSWD structure,
 * and the other for GGROUPS structure. The first one is called PASSWD database,
 * and the other is GROUP database.
 *
 * The file also presents the interfaces to store/fetch/modify user's private 
 * information into databases.
 */

#ifndef _XOS_DBAUX_H_
#define _XOS_DBAUX_H_

#include <sys/types.h>
#include <db.h>
#include "xos.h"

#define MAXFIELD	DATAFIELD_MAXLEN/**< Maxmiun length of string space */
#define MAXDATABUF 	MAXFIELD + 1



#define	DEBUG_INFO	1

/** @brief global identity token of Grid user's account.
 */
typedef struct _global_identity_token {
	char g_dn[MAXFIELD];
			  /**< DN of Grid user */
	char g_vo[MAXFIELD];
			  /**< VO of Grid user */
	char g_role[MAXFIELD];
			  /**< ROLE of Grid user */
	char g_guid[MAXFIELD];
			/**< GUID of Grid user*/

} GIDTOKEN;

/** @brief local identity token of Grid user's account.
 */
typedef struct _local_identity_token {
	char g_mappedname[MAXFIELD];
				  /**< local mapped name */
	uid_t g_mappeduid;	  /**< local mapped uid */
	gid_t g_mappedgid;	  /**< local mapped primary gid */

} LIDTOKEN;

/** @brief GPASSWD of Grid user.
 * It is also a relationship between global idnetity token and 
 *local identity token.
 */
typedef struct _grid_passwd {
	GIDTOKEN g_idtoken;
			 /**< global identity token of account */
	LIDTOKEN l_idtoken;
			 /**< local identity token of account */

	char g_shell[MAXFIELD];
			 /**< local shell of grid user */
	char g_home[MAXFIELD];
			 /**< local home directory */
	long g_time;	 /**< time stamp */

	int g_mount;	/**< mount counter */

} GPASSWD;

/** @brief global idenity token of grid user's group
 */
typedef struct _global_group_token {
	char g_vo[MAXFIELD];  /**< VO of grid user */
	char g_subgroup[MAXFIELD];
			      /**< Subgroups of grid user*/

} GGRPTOKEN;

/** @brief local identity token of grid user's group
 */
typedef struct _local_group_token {
	char g_mappedname[MAXFIELD];
			       /**< local group name */
	gid_t g_mappedgid;     /**< local group id */
	char g_members[MAXFIELD];
			       /**< members in group */

} LGRPTOKEN;

/** @brief GGROUPS of Grid user.
 */
typedef struct _grid_groups {
	GGRPTOKEN g_grptoken;
			   /**< global identity token of group */
	LGRPTOKEN l_grptoken;
			   /**< local identity token of group */
	long g_time;	   /**< time stamp */

} GGROUPS;

/** Special function for data processing in GPASSWD
 * The function can be defined by outside function with
 * same input and output, to deal with data item in
 * GPASSWD database.
 *
 */

typedef int (*OPTPWDFUNC) (GPASSWD *, void *, void *);

/** Special function for data processing in GGROUPS
 * The function can be defined by outside function with
 * same input and output, to deal with data item in
 * GPASSWD database.
 *
 */
typedef int (*OPTGRPFUNC) (GGROUPS *, void *, void *);

/*Interfaces of PASSWD database*/

/**create a new gpasswd entiry */
GPASSWD *gpwd_new();
/**Store GPASSWD struct to PASSWD database */
int bdb_storedben_pwd(GPASSWD *);
/**Print data in GPASSWD struct as readable format */
void bdb_printdata_pwd(GPASSWD *);
/**Print all the data in GPASSWD database */
void bdb_printdb_pwd(void);
/**Fetch GPASSWD struct by given user's DN */
char *bdb_getdbname_pwd(char *, GPASSWD *);
/**Fetch GPASSWD struct by given user's local uid */
char *bdb_getdbuid_pwd(uid_t, GPASSWD *);
/**Fetch GPASSWD struct by given user's local account name */
char *bdb_getdbuname_pwd(char *, GPASSWD *);
/**Fetch GPASSWD struct by given user's global identity token */
char *bdb_getdbgtk_pwd(GIDTOKEN *, GPASSWD *);
/**Fetch GPASSWD struct by given user's local identity token */
char *bdb_getdbltk_pwd(LIDTOKEN *, GPASSWD *);
/**Delete GPASSWD struct in PASSWD database by given user's global identity token */
int bdb_deldbgtk_pwd(GIDTOKEN *);
/**Dispose the given data in database by self-defined function */
int bdb_operatedata_pwd(GIDTOKEN *, OPTPWDFUNC);
/**Dispose all the  data in database by self-defined function*/
int bdb_operatedb_pwd(void *, void *, OPTPWDFUNC);

/*Interface of GROUP database*/

/**create a new ggroups entiry */
GGROUPS *ggrp_new();
/**Store GGROUPS struct to GROUP database */
int bdb_storedben_grp(GGROUPS *);
/**Print data in GGROUPS struct as readable format */
void bdb_printdata_grp(GGROUPS *);
/**Print all the data in GGROUPS database */
void bdb_printdb_grp(void);

char *bdb_getdbname_grp(char *, GGROUPS *);
/**Fetch GGROUPS struct by given user's local gid */
char *bdb_getdbgid_grp(gid_t, GGROUPS *);

char *bdb_getdbgname_grp(char *, GGROUPS *);
/**Fetch GGROUPS struct by given user's global identity token */
char *bdb_getdbgtk_grp(GGRPTOKEN *, GGROUPS *);
/**Fetch GGROUPS struct by given user's local identity token */
char *bdb_getdbltk_grp(LGRPTOKEN *, GGROUPS *);
/**Delete GGROUPS struct in PASSWD database by given user's global identity token */
int bdb_deldbgtk_grp(GGRPTOKEN *);
/**Dispose the given data in database by self-defined function */
int bdb_operatedata_grp(GGRPTOKEN *, OPTGRPFUNC);
/**Dispose all the  data in database by self-defined function*/
int bdb_operatedb_grp(void *, void *, OPTGRPFUNC);

/*internal functions*/

/**Create random string key */
void create_sranddbkey(char *, int);
/**Print error as readable string */
void print_error(int, const char *);
/** Print debug information */
void print_debug(const char *fmt, ...);
/**Initiate DBT structure of key and data */
void init_DBT(DBT *, DBT *);
/**Pack string to buffer */
size_t pack_string(char *, char *, size_t);

#endif /*_XOS_DBAUX_H_*/
