/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_interfaces.h
 * @brief common interfaces of node-level system
 *
 * The codes implemented the common interfaces with node-level 
 * NSS/PAM module and AMS module.
 *
 * The part of code is under development.
 *
 */


#ifndef _XOS_INTERFACES_
#define _XOS_INTERFACES_

#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include "xos_key.h"		/* header of encryptation */
#include "xos_sslutils.h"	/* header of certificate & proxy disposal */
#include "xos_rquota.h"		/* header of resouce quota disposal */

/** return value: success */
#define XOS_SUCCESS	0

/** return value: fail */
#define XOS_ERROR	-1

/** Data structure of proxy.
 * It uses the SSL_CREDENTIALS defined in xos_sslutils.h
 * as the carrier. 
 */
#define XOS_CREDS 	SSL_CREDENTIALS

/** Data type of asymmetric key in Xtreemos 
 */
#define XOS_ASYMKEY 	Key

/** Data type of symmetric key in Xtreemos */
#define XOS_SYMKEY	SymKey

/** Data type of resouces quota in Xtreemos */
#define XOS_RQUOTA	ResQuota


/*--------------------------------*/
/* Certificate-related interfaces */
/*--------------------------------*/

/* This part of codes should be reconstructed as 
 * 
 * int xos_function_name(struct certificate_struct, type other_params)
 *
 * The return value of integer shows whether the function is OK after
 * operation; and certificate_struct is data structure holding the 
 * following fields:
 *  - new DN
 *  - expiry time (in seconds)
 *  - attributes
 *  - policy file
 *  - certificate file
 *  - key file
 *  - etc
 * The function gets the input parameters from certificate_struct, and
 * does the proper operations.
 * 
 * Other parameters of the function can be log_level, buffer pointer or
 * status information.
 */


/** Initialize the proxy.
 * The function creates the first proxy certificate by user's 
 * certificate and signed it by user's private key. User's 
 * certificate and private key are given by applying from CA.
 */
int xos_proxy_init(char *usercert, char *userkey, char *proxyfile);

/** Delegate next proxy certificate by current proxy.
 * The function implements delegation from a proxy to another proxy, 
 * and set their lifetime.
 */
int xos_proxy_delegate(char *current_proxy, char *next_proxy, int days);

/** Destroy the proxy.
 * 
 */
int xos_proxy_destroy(char *proxyfile);

#if 0
/* (disabled)
 * If the library is used to built CA, then the following
 * functions should be implemented.
 */

/** Generate a certificate request */
int xos_cert_req();

/** Create a certificate */
int xos_cert_create();
#endif

/** Verify (proxy) certificate. 
 * The function implements the codes for verifying the user's 
 * certificate (including proxy certificate). 
 * @param cadir The path of CA certificate directory.
 * @param proxyfile The buffer holding (proxy) certificate to
 *		be verified.
 * @param loglevel The level of loging information
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
#define xos_cert_verify(cadir, proxyfile,loglevel) \
	xos_cdaproxy_verify(cadir,proxyfile,loglevel)

int xos_cdaproxy_verify(char *cadir, char *proxyfile, int loglevel);


int xos_cdacert_verify(unsigned char *cert_buf, char *cda_dir, char *cda_file);

/** Set new attributes in certificate.
 * @param proxy_file The path to proxy file.
 * @param attrs the attributes to embed in proxy certificate.
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_attributes_set(char *proxy_file, char *attrs);

/** Fetch attributes from certificate.
 * @param proxy_file The path to proxy file 
 * @param attrs the attributes to embed in proxy certificate.
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_attributes_load(char *proxy_file, char *attrs);


/** Fetch attributes from certificate.
 * @param cert the X509 certiificates.
 * @param attrs the attributes to embed in proxy certificate.
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_attributes_fetch(X509 * cert, char *attrs);

int xos_cert_attributes_fetch_ext(X509 * cert, char *exts);

int xos_cert_attributes_fetch_attr(char *ext, char *attr_label,
				   char *attr_value);

/** Set lifetime of certificate.
 * The function will be called when the proxy or certificate created 
 * first time.
 * @param proxy_buffer the pointer of proxy buffer
 * @param seconds the time (in seconds) to expire proxy
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_lifetime_set(char *proxy_buffer, long seconds);

/** Get the lifetime of certificate
 * (disable) under development.
 */
int xos_cert_lifetime_get(X509 * proxy_cert, time_t * not_after);

/** Get subject from certificate
 * The function fetchs the subject from /a cert and store in buffer.
 * @param cert the certificate pointer to be fetched subject 
 * @param buffer The buffer to store subject
 * @param buflen the length of subject
 * @return XOS_SUCCESS if success, or XOS_ERROR.
 */
int xos_cert_subject_get(X509 * cert, char *buffer, int buflen);

/** Set subject to certificate
 */
int xos_cert_subject_set();


/*------------------------*/
/* key-related interfaces */
/*------------------------*/


/** Generate asymmetric key pair.
 * @param type key type returned by xos_asymkey_keytype()
 * @param bits
 * @return pointer of XOS_ASYMKEY struct.
 */
XOS_ASYMKEY *xos_asymkey_gen(int type, u_int bits);

/** Get public key from key pair
 * @param key key pair generated from xos_asymkey_gen()
 * @return pointer of XOS_ASYMKEY if success, or NULL
 */
XOS_ASYMKEY *xos_asymkey_getpubkey(const XOS_ASYMKEY * key);

/** Get internal key type from readable name
 * @param name "rsa" or "dss"
 * @return integer of internal key type
 */
int xos_asymkey_keytype(char *name);

/** Sign with asymmetric key
 * @param key asymmtric key presented by xos_asymkey_gen()
 * @param sigp signature buffer to hold the signature.
 * @param lenp the length of signature buffer.
 * @param data the buffer holding the data
 * @param datalen the length of data 
 * @return XOS_SUCCESS if success, or XOS_ERROR 
 */
int xos_asymkey_sign(const XOS_ASYMKEY * key,
		     u_char ** sigp, u_int * lenp,
		     const u_char * data, u_int datalen);

/** Verify with asymmetric key
 * @param key asymmtric key presented by xos_asymkey_gen()
 * @param sigp signature buffer to hold the signature.
 * @param lenp the length of signature buffer.
 * @param data the buffer holding the data
 * @param datalen the length of data 
 * @return XOS_SUCCESS if success, or XOS_ERROR 
 */
int xos_asymkey_verify(const XOS_ASYMKEY * key,
		       const u_char * signature, u_int signaturelen,
		       const u_char * data, u_int datalen);

/** Generate symmtric key
 * @param k symmetric key to generate
 * @return XOS_SUCCESS if success, or XOS_ERROR 
 */
int xos_symkey_gen(XOS_SYMKEY * k);

/** Encrypt data with symmetric key
 * @param dataToEncrypt the pointer of data to be encrypted.
 * @param dataToEncryptLen the length of data
 * @param encryptedData the pointer of encrypted data
 * @param encryptedDataLen the length of encrypted data
 * @param k the symmetric key used in crypt
 * @return XOS_SUCCESS if success, or XOS_ERROR 
 */
int xos_symkey_encrypt(unsigned char *dataToEncrypt, int dataToEncryptLen,
		       unsigned char *encryptedData, int *encryptedDataLen,
		       XOS_SYMKEY * k);

/** Decrypt data with symmetric key
 * @param dataToEncrypt the pointer of data to be encrypted.
 * @param dataToEncryptLen the length of data
 * @param encryptedData the pointer of encrypted data
 * @param encryptedDataLen the length of encrypted data
 * @param k the symmetric key used in crypt
 * @return XOS_SUCCESS if success, or XOS_ERROR 
 */
int xos_symkey_decrypt(unsigned char *dataToDecrypt, int dataToDecryptLen,
		       unsigned char *decryptedData, int *decryptedDataLen,
		       XOS_SYMKEY * k);

/** Print symmetric key
 * @param k symmetric key to print
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_symkey_prt(XOS_SYMKEY * k);



/*------------------------*/
/* I/O-related interfaces */
/*------------------------*/


/** Create a new struct 
 */
XOS_CREDS *xos_creds_new();

/** Release the creds struct
 * @param creds the pointer of XOS_CREDS struct
 */
void xos_creds_free(XOS_CREDS * creds);

/** Load certificate from file
 * @param creds the struct of XOS_CREDS to hold certificate
 * @param the path of certificate file
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_loadfromfile(XOS_CREDS * creds, const char *path);


/** Save certficate from buffer to disk file
 * @param creds the struct of XOS_CREDS to hold private key
 * @param path the path of private key file
 * @param pass_phrase the callback function pointer
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_savetofile(XOS_CREDS * creds,
			const char *path, const char *pass_phrase);;

/** Load private key from file
 * @param creds the struct of XOS_CREDS to hold private key
 * @param path the path of private key file
 * @param pass_phrase the callback function pointer
 * @param pass_phrase_prompt The password prompt string
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_prvkeyfromfile(XOS_CREDS * creds,
			    const char *path,
			    const char *pass_phrase,
			    const char *pass_phrase_prompt);


/** Load certificates file to buffer
 * @param path the path to disk file of certificate
 * @param buffer The pointer of buffer to holding certificate
 * @param pbuffer_len Length of certificate
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_cert_buffromfile(const char *path,
			 unsigned char **pbuffer, int *pbuffer_len);

/** Load proxy from file
 * The function loads the proxy from file and constructs the 
 * proxy certificate, private key, and certificate chain.
 * @param creds the XOS_CREDS struct to hold proxy certificate
 * @param path the path of proxy file
 * @param pass_phrase the callback function to deal with private key
 *		in proxy file
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_proxy_loadfromfile(XOS_CREDS * creds,
			   const char *path, const char *pass_phrase);


/** Load proxy from buffer
 * @param creds the proxy struct should be allocated previously
 * @param buffer Buffer is stored in certificates
 * @param buffer_len the space length of buffer
 * @param pass_phrass if certificate is crypted with password, the
 *		pass_phrass is prompt string to enter user's password
 * @param XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_proxy_loadfrombuffer(XOS_CREDS * creds,
			     const unsigned char *buffer, int buffer_len,
			     const char *pass_phrase);

int xos_cert_loadfrombuffer(unsigned char *buffer, int buffer_length,
			    XOS_CREDS * creds);
#if 0

/*----------------------------*/
/* Service-related interfaces */
/*----------------------------*/


/* (disabled)
 * The codes here should be built based on
 * the four services defined in WP3.5.
 */


/** Get identity from Identity service.
 * The identity service is one of the security services 
 * defined in WP3.5 specification. 
 */
int xos_service_identity_get();

/** Get attributes from Attributes service
 */
int xos_service_attr_get();

/** Get policy from policy service
 */
int xos_service_prolicy_get();

#endif


#if 0

/*----------------------------*/
/* Mapping-related interfaces */
/*----------------------------*/

/* (disabled)
 * The part of codes has been implemented 
 * in xos_ams.h as a client end for connectting 
 * with AMS and get mapping information from AMS.
 * Developers can call the interfaces by including
 * xos_ams.h.
 *
 */


/** Map the global user to local one.
 */
int xos_ams_mapping(char *mapped_dn, char *mapped_vo, char *mapped_role,
		    char *mapped_subgroup, XOS_GPASSWD * gpwd,
		    XOS_GGROUPS * primarygrp);


/** Get the global user according to local counter part
 */
int xos_ams_imapping();
#endif


/*--------------------------*/
/* Quota-related interfaces */
/*--------------------------*/


/** Initialize the structure of resouce quota.
 * @param rquota The struct of quota should be
 *	defined and initialized before usage.
 * @return XOS_SUCCESS
 */
int xos_rquota_init(XOS_RQUOTA * rquota);

/** Setup the quota limitation.
 * The function will store the rquota struct to
 * vo_acconf.conf which is read by PAM module.
 * @param rquota the struct of resource quota.
 * @return XOS_SUCCESS if success, or XOS_ERROR
 */
int xos_rquota_setup(XOS_RQUOTA * rquota);

/** Clear the quota limition
 * The function will clear the user's rquota limitation
 * in vo_acconf.conf read by PAM module.
 * @param rquota The struct of reaouces quota to be cleared
 * @return XOS_SUCCESS if success, or XOS_ERROR 
 */
int xos_rquota_clear(XOS_RQUOTA * rquota);

int xos_rquota_modify(XOS_RQUOTA * rquota, int flags);

#if 0
/** Interpret the policy and translate to rquota struct.
 * The function is inputed a certificate_struct mentioned above
 * and a XOS_RQUOTA. Then, it interprets the policy and sets the
 * fields in XOS_RQUOTA, which will be writed in access file read
 * by PAM module.
 *
 * (disabled)
 * Becaucse we have not defined the policy in WP3.5
 */
int xos_rquota_interpret();
#else

/** Interpret the attributes - ROLE 
 * The function just presents a simple translation from 
 * attibute - ROLE to quota limitation.
 * @param user_role the ROLE attribute fetched from user's certificate.
 * @param rquota The quota struct holding limitation information
 *		interpreted by predefined policy mapping configuration.
 * @return XOS_SUCCESS if success, or XOS_ERROR.
 */
int xos_rquota_interpret(char *user_role, XOS_RQUOTA * rquota);

#endif

#endif /*_XOS_INTERFACES_*/
