/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */


/** @file xos_sslutils.h
 * @brief Utility Library about security, authentication, etc.
 *
 * The file defines the interfaces about security. Most of them
 * focues on encrytation/decryptation, certificat/proxye
 * verifying and delegation, and siguature, etc.
 *
 * The codes implemented here are based on OpenSSL.
 */

#ifndef _XOS_SSLUTILS_H_
#define _XOS_SSLUTILS_H_

#include <openssl/ssl.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/md5.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/rand.h>
#include <openssl/des.h>
#include <openssl/rsa.h>
#include <openssl/dsa.h>
#include <openssl/x509_vfy.h>
#include "xos_key.h"
#include "buffer.h"
#include "xos_utils.h"
/*
 * Returns values for functions
 */
/** callback function */
#define PEM_NO_CALLBACK NULL,NULL
/** callback function */
#define PEM_CALLBACK(func)  func,NULL

/** return value of success */
#define SSL_SUCCESS	1
/** return value of error */
#define SSL_ERROR	0

/** @brief data structure for proxy.
 *  The proxy is defined by RFC3820.
 */
struct _ssl_credentials {
	X509 *certificate;	     /**< current certificate of proxy*/
	EVP_PKEY *private_key;	     /**< private key of proxy */
	STACK *certificate_chain;	   /**< certificate chain of issuer's certificates in proxy */

};

/** struct of proxy */
typedef struct _ssl_credentials SSL_CREDENTIALS;

/** create a new credentials struct
 * @return the pointer of SSL_CREDENTIALS struct if success, or NULL
 */
SSL_CREDENTIALS *ssl_credentials_new();


/** Free the content of SSL_CREDENTIALS struct
 * @param creds the pointer holding the content, which should be preacllocated
 *        in other context.
 */
void xosssl_credsfreecontents(SSL_CREDENTIALS * creds);

/** Transfor the key type
 * @param pk  the key with the type of EVP_PKEY.
 * @return struct key defined in key.h
 */
Key *pemkey_transfor(EVP_PKEY * pk);

/** Load certificate from disk file
 * @param cerds the SSL_CREDENTIALS buffer for storing certificate.
 *             It shold be predefined in outside function and allocated
 *              the space.
 * @param path the path of pem-format file. The file must be standard
 *             pem foramt.
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_loadcertfromfile(SSL_CREDENTIALS * creds, const char *path);

/** Get subject from certificate
 * @param cert The certificate of X509 type.
 * @param buffer The buffer to store subject. The subject is string.
 * @param buflen The length of buffer.
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_getsubjectfromcert(X509 * cert, char *buffer, int buflen);

/** Load proxy to SSL_CREDENTIALS struct from disk file
 * @param creds The struct buffer to hold proxy content
 * @param path The path of proxy file. The file must be pem-format.
 * @param pass_phrase The prompt of password function. When the pem file
 *                 has been encrypted, the prompt would be called to
 *                 input  password.
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_loadproxyfromfile(SSL_CREDENTIALS * creds,
			     const char *path, const char *pass_phrase);
/** Get public key from certificate
 * @param cert The certifcate with X509 type
 * @return Key of public key. The key is tranfored to the type of
 *         struct key defined in key.h
 */
Key *xosssl_getpublickeyfromcert(X509 * cert);

/** Load the content of pem-format file to buffer. The pem-file can be
 * stored in certificate, proxy, public key or private key.
 * @param path the path of pem-format file.
 * @param pbuffer The pointer of buffer to return. The value should be
 *                predefined.
 * @param pbuffer_len The pointer of buffer length to return. The value
 *                returned by function is masked by the buffer length
 *                read from file.
 * @return SSL_SUCCESS if sucess, or SSL_ERROR
 */
int xosssl_bufferfromfile(const char *path, unsigned char **pbuffer,
			  int *pbuffer_len);

/** Parse the content of buffer and return the proxy structure.
 * The buffer can be the one after called the \a xosssl_locaproxyfromfile()
 * or passed by other routines. The function analyzes the content in buffer
 * and reconstructs the proxy in SSL_CREDETIALS struct.
 *
 * @param creds The SSL_CREDENTIALS struct to hold proxy content, which
 *            should be preallocated space in outside function.
 * @param buffer The buffer holding the content of proxy
 * @param buffer_len The length of content in buffer
 * @param pass_phrase The prompt string for prompting input password
 *
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_proxyfrompem(SSL_CREDENTIALS * creds,
			const unsigned char *buffer, int buffer_len,
			const char *pass_phrase);

/** Get private key from file.
 * The private key stored in disk file usually encrypted by user's password.
 * The function will prompt the caller to input the password, when it tries
 * to read the private key.
 *
 * @param creds The SSL_CREDENTIALS struct to store private key.
 * @param path the path of file stored private key
 * @param pass_phrase The prompt of password
 *
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_getprivatekeyfromfile(SSL_CREDENTIALS * creds,
				 const char *path,
				 const char *pass_phrase,
				 const char *pass_phrase_prompt);


int
xosssl_proxytofile(SSL_CREDENTIALS * proxy_creds,
		   const char *path, const char *pass_phrase);

int xosssl_proxydestroy(const char *proxyfile);

/** Get expire time from proxy.
 * The function fetchs the boundary of expired time from proxy file.
 * Moreoften, it should be from buffer holding proxy, not from file
 * in disk.
 *
 * @param path the path of proxy file (currently).
 * @param not_before the low boundary of expire time
 * @param not_after the high boundary of expire time
 *
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_gettimes(const char *path, time_t * not_before,
		    time_t * not_after);

/** encrypt data with symmetric key
 * @param dataToEncrypt data before encrypting
 * @param datoToEncryptLen the length of encryted data before encrypting.
 * @param encryptedData data after encrypting
 * @param encryptedDataLen the length of cncrypted data after encrypting.
 * @param k symmetric key used in this encryptation. The sysmmetric key should
 *          be created before calling this routime.
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_encryptWithSymKey(unsigned char *dataToEncrypt,
			     int dataToEncryptLen,
			     unsigned char *encryptedData,
			     int *encryptedDataLen, SymKey * k);

/** Decrypt data with symmetric key
 * @param dataToDecrypt encrypted data pointer to be decrypted
 * @param dataToDecryptLen the length of data to be decrypted
 * @param decryptedData data after decrypting
 * @param decryptedDataLen the length of decrypted data
 * @param k symmetric key
 *
 * @return SSL_SUCCESS if success, or SSL_ERROR
 */
int xosssl_decryptWithSymKey(unsigned char *dataToDecrypt,
			     int dataToDecryptLen,
			     unsigned char *decryptedData,
			     int *decryptedDataLen, SymKey * k);


/** verify proxy delegated by CDA-issued certificate
 * @param cddir the path of CA certificate
 * @param proxyfile the buffer holding the proxy 
 * @param loglevel the level of print 
 * @return SSL_SUCCESS if success, or SSL_ERROR
 * */
unsigned long xosssl_cdaproxy_verify(char *cadir, char *proxyfile, int loglevel);

#define verifyAll(cadir,proxyfile,loglevel)  \
			xosssl_cdaproxy_verify(cadir, proxyfile, loglevel)

/** parse cerrtificate from buffer
 * @param buffer the buffer holding the certificate.
 * @param buffer_length the lengh of buffer
 * @param creds the credential structure.
 * @return SSL_SUCCESS if success, or SSL_ERROR
 * */
int xosssl_certfrombuffer(unsigned char *buffer, int buffer_length,
			  SSL_CREDENTIALS * creds);

/** verify CDA-issured certificate
 * @param cert the certificate
 * @param cda_dir the path of CDA root certificate
 * @param cda_file the name of CDA root certificate
 * @return SSL_SUCCESS if success, or SSL_ERROR
 * */
int xosssl_cdacert_verify(X509 *cert, char *cda_dir, char *cda_file);

#endif				/* _XOS_SSLUTILS_H_ */
