/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_utils.h
 * @brief Definition of DEBUG() and DEBUG_ASSERT() macros.
 *
 * This file defines some macros that can be used for debugging purposes.
 */
#ifndef _XOS_UTILS_H
#define _XOS_UTILS_H

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>		/* LOG_* */
#include <sys/types.h>
#include <errno.h>
#include <stdarg.h>

/**
 * \def D_EMPH
 * ANSI sequence to emphasize text (empty string if D_COLOR_ENABLED is not
 * defined)
 * \def D_RESET
 * ANSI sequence to reset emphasized text (empty string if D_COLOR_ENABLED is
 * not defined)
 */
#ifdef D_COLOR_ENABLED
#  define D_EMPH "\033[31m"
#  define D_RESET "\033[0m"
#else				/* D_COLOR_ENABLED */
#  define D_EMPH ""
#  define D_RESET ""
#endif				/* D_COLOR_ENABLED */

/** @brief Debug messages.
 *
 * This macro outputs debug messages, indicating the file and line number
 * where the message was generated.
 * @param args List of arguments (in printf format)
 */
#define DEBUG(args...) xos_debug("debug",__FILE__, __LINE__, args)

/** @brief Issue a warning if @a expr is false.
 *
 * This function can be used to check for bugs (i.e. a pointer must not be
 * NULL). If condition @a expr is not true, a warning message will be printed.
 * @param expr Expression to evaluate (should not contain function calls to
 *             avoid side effects).
 */
#define DEBUG_ASSERT(expr) do { \
	if (!(expr)) { \
		DEBUG("assertion `" #expr "' failed in function %s" D_EMPH \
			"(bug?)" D_RESET, __FUNCTION__); \
	} \
} while(0)

/** @brief Check whether the current user is a global user.
 *
 * This function can be used by applications to check whether it is running
 * under the identity of a global user.
 * @retval 0 User is a local user
 * @retval !0 User is a global user
 * 
 * @todo This function only checks that the environment variable XOS_ENV is
 * defined. In future versions, a kernel key should be retrieved and its
 * validity checked.
 */
int xos_is_user_global(void);

/** @brief Output a debug message.
 *
 * This function outputs a debug message on stderr, or on the system log if
 * _DEBUG_SYSLOG is defined. This function is intended to be called via the
 * macro DEBUG(), which will initialize the parameters file and line.
 * @param file Source file that caused the debug message (__FILE__)
 * @param line Line number in source file (__LINE__)
 * @param fmt Formatted debug message
 */
void xos_debug(const char *level, const char *file, int line, const char *fmt, ...);

/** @brief Log a message to the system log.
 *
 * This function outputs a debug message to the system log.
 * @param option The option parameter for openlog (i.e. LOG_PID)
 * @param facility Used to specify what type of program is logging the message
 * @param level Message level (i.e. LOG_DEBUG)
 * @param format Formatter message to be logged
 */
void xos_log(int option, int facility, int level, const char *format, ...);

/** @brief Exist when meet fatal error.
 *
 * The function shrows out a debug information and exist current thread,
 * if the error is thought to be a fatal error.
 * @param fmt List of arguments in printf format.
 */
void fatal(const char *fmt, ...);


#define hd_log(format, level, ...) do {                               \
/*
                fprintf(stderr, "[%s] \t%s:%d - " format "\n", \
                        level, __FUNCTION__, __LINE__, ##__VA_ARGS__); \
*/		xos_debug(level, __FUNCTION__, __LINE__, format,##__VA_ARGS__); \
        } while (0)

#define hd_log_error(format, ...) hd_log(format, "error", ##__VA_ARGS__);
#define hd_log_warning(format, ...) hd_log(format, "warning", ##__VA_ARGS__);
#define hd_log_info(format, ...) hd_log(format, "info", ##__VA_ARGS__);
#define hd_log_debug(format, ...) hd_log(format, "debug", ##__VA_ARGS__);
#define hd_log_trace(format, ...) hd_log(format, "trace", ##__VA_ARGS__);
#define hd_log_syserror(format, ...) do { \
                fprintf(stderr, "[syserr] \t%s:%d: %s - " format "\n", \
                        __FUNCTION__, __LINE__, strerror(errno), \
                        ##__VA_ARGS__);                                 \
        } while (0)

#define MAXTIMELEN 47;
#define ERRNO_FORMAT "%d (%s)"

#endif				/* _XOS_UTILS_H */
